@@ -1,5 +1,5 @@
 
-# $Id: Wizard.pm,v 2.66 2007/10/20 03:34:31 martinthurn Exp $
+# $Id: Wizard.pm,v 2.71 2007/11/16 21:28:58 martinthurn Exp $
 
 package Tk::Wizard;
 
@@ -9,9 +9,9 @@ use warnings;
 use constant DEBUG_FRAME => 0;
 
 our
-$VERSION = do { my @r = ( q$Revision: 2.66 $ =~ /\d+/g ); sprintf "%d." . "%03d" x $#r, @r };
+$VERSION = do { my @r = ( q$Revision: 2.71 $ =~ /\d+/g ); sprintf "%d." . "%03d" x $#r, @r };
 
-my $sdir = ($^O =~ m/MSWin32/i) ? 'Folder' : 'Directory';
+my $sdir = ($^O =~ m/MSWin32/i) ? 'folder' : 'directory';
 my $sDir = ucfirst $sdir;
 
 =head1 NAME
@@ -26,6 +26,7 @@ use Data::Dumper;
 use File::Path;
 use File::Slurp;
 use File::Spec::Functions qw( rootdir );
+use Scalar::Util qw( reftype );
 use Tk;
 use Tk::DialogBox;
 use Tk::DirTree;
@@ -724,12 +725,18 @@ sub initial_layout
       {
       $self->Photo( "sidebanner", -data => $$im );
       }
+    my $sBG = $self->_on_first_page ? 'white'
+            : $self->_on_last_page ? 'white'
+            : $self->{background};
     $self->{left_object} = $self->Label(
                                         -image => "sidebanner",
+                                        -anchor => "n",
+                                        -background => $sBG,
                                        )->pack(
-                                               -side => "top",
                                                -anchor => "n",
+                                               -fill => 'y',
                                               );
+    DEBUG_FRAME && $self->{left_object}->configure(-background => 'blue');
     } # if 95 or first page
   else
     {
@@ -764,7 +771,7 @@ sub render_current_page
     } # if
   if ($self->_on_first_page || $self->_on_last_page)
     {
-    $self->{left_object}->pack( -side => "left", -anchor => "w" );
+    $self->{left_object}->pack( -side => "left", -anchor => "n", -fill => 'y' );
     if ( $self->{-style} ne '95' )
       {
       $frame_pack{-expand} = 1;
@@ -1274,7 +1281,11 @@ sub _NextButtonEventCycle
   # warn " DDD start, NBEC counter is now $self->{_inside_nextButtonEventCycle_}\n";
   # If there is more than one pending invocation, we will reinvoke
   # ourself when we're done:
-  return if (1 < $self->{_inside_nextButtonEventCycle_});
+  if (1 < $self->{_inside_nextButtonEventCycle_})
+    {
+    warn "   called recursively, bail out\n" if $self->{-debug};
+    return;
+    } # if
   if ( _dispatch( $self->cget( -preNextButtonAction ) ) )
     {
     warn " DDD preNextButtonAction says we should not go ahead\n" if $self->{-debug};
@@ -1300,6 +1311,7 @@ sub _NextButtonEventCycle
     } # if last page
   # Advance the wizard page pointer and then adjust the navigation buttons.
   # Redraw the frame when finished to get changes to take effect.
+  warn " DDD   advance to next page\n" if $self->{-debug};
   $self->_page_forward;
   $self->render_current_page;
   # print STDERR " DDD this is before _dispatch postNextButtonAction\n";
@@ -1309,7 +1321,7 @@ sub _NextButtonEventCycle
     goto ALL_DONE_NBEC;
     } # if
  ALL_DONE_NBEC:
-  # warn " DDD all done, NBEC counter is now $self->{_inside_nextButtonEventCycle_}\n";
+  warn " DDD all done, NBEC counter is now $self->{_inside_nextButtonEventCycle_}\n" if $self->{-debug};
   $self->{_inside_nextButtonEventCycle_}--;
   $self->_NextButtonEventCycle('no increment') if $self->{_inside_nextButtonEventCycle_};
   } # _NextButtonEventCycle
@@ -1511,30 +1523,12 @@ sub _page_dirSelect
                                if ($new_name) {
                                  $new_name =~ s/[\/\\]//g;
                                  $new_name = ${ $args->{-variable} } . "/$new_name";
-                                 if ( !mkdir $new_name, 0777 ) {
-                                   my $msg;
-                                   if ( $! =~ /Invalid argument/i ) {
-                                     $msg = "The $sdir name you supplied is not valid.";
-                                     }
-                                   elsif ( $! =~ /File Exists/i ) {
-                                     $msg = "A $sdir with that name already exists.";
-                                     }
-                                   else {
-                                     $msg = "The $sdir could not be created:\n\n\t'$!'";
-                                     }
-                                   $self->messageBox(
-                                                     '-icon'  => 'error',
-                                                     -type    => 'ok',
-                                                     -title   => 'Could Not Create $sDir',
-                                                     -message => $msg,
-                                                    );
-                                   } # if
-                                 else {
+                                 if ($self->_cb_try_create_dir($new_name))
+                                   {
                                    ${ $args->{-variable} } = $new_name;
-                                   # print STDERR " DDD   4 DirTree->configure(-directory=>$new_name)\n";
                                    $dirs->configure( -directory => $new_name );
                                    $dirs->chdir($new_name);
-                                   } # else
+                                   } # if
                                  } # if new_name
                                }, # end of -command sub
                             )->pack( -side => 'right',
@@ -1558,18 +1552,12 @@ sub _page_dirSelect
                            -ipadx => 5,
                          );
     } # if
-  # Add the user's requested directory:
-  $dirs->configure(-directory => ${$args->{-variable}}) if (${$args->{-variable}} ne '');
-
-  # The DirTree itself can take a long time to draw:
-  $self->{wizardFrame}->update;
-  $self->idletasks;
-  $self->update;
-  foreach my $d (&$_drives) {
+  foreach my $d (&$_drives)
+    {
     # Try to prevent GUI freeze:
     $self->idletasks;
     $self->{wizardFrame}->update;
-    # $self->update;
+    $self->update;
     ($d) =~ /^(\w+:)/;
     if (
         $args->{-nowarnings}
@@ -1591,6 +1579,8 @@ sub _page_dirSelect
       $dirs->configure( -directory => $d );
       }
     } # foreach
+  # Make the user's requested directory appear as the default (?):
+  $dirs->chdir(${$args->{-variable}}) if (${$args->{-variable}} ne '');
   $self->Unbusy;
   return $frame;
   } # _page_dirSelect
@@ -1731,13 +1721,13 @@ sub _page_fileSelect
                                  # getOpenFile will croak if the
                                  # -initialdir we give it does not
                                  # exist:
-                                 my $sDir = $args->{-directory};
-                                 if (! -d $sDir)
+                                 my $sDirInit = $args->{-directory};
+                                 if (! -d $sDirInit)
                                    {
-                                   $sDir = &File::Spec::rootdir;
+                                   $sDirInit = &File::Spec::rootdir;
                                    } # if
                                  my $sFname = $frame->getOpenFile(
-                                                                  -initialdir => $sDir,
+                                                                  -initialdir => $sDirInit,
                                                                   -title      => $args->{-title},
                                                                  );
                                  ${ $args->{-variable} } = $sFname if $sFname;
@@ -2068,6 +2058,152 @@ sub _page_multiple_choice
     } # _page_multiple_choice
 
 
+=head2 addSingleChoicePage
+
+Allow the user to make one choice from among several options
+(i.e. a group of radio buttons).
+Each choice sets a variable passed as reference to this method.
+
+Accepts the usual parameters plus:
+
+=over 4
+
+=item -relief
+
+For the radio buttons - see L<Tk::options>.
+
+=item -variable
+
+Reference to a variable that will contain the result of the choice.
+Croaks if none supplied.  Your -variable will contain the -value of the
+radio button that is selected when the user clicks "Next".
+
+=item -choices
+
+A reference to an array of hashes with the following fields:
+
+=over 4
+
+=item -title
+
+Title of the option, will be rendered in bold
+
+=item -subtitle
+
+Text rendered smaller beneath the title
+
+=item -value
+
+This value will be placed in your -variable variable if this button is
+selected
+
+=item -selected
+
+Pass a true value to specify that this radio should initially appear
+selected.  If none of the choices have -selected, then the first
+choice will be selected by default.
+
+=back
+
+Here is an example of what the -choices parameter should look like:
+
+  $wizard->addSingleChoicePage(
+    -title => 'Another toy example',
+    -text => 'Choose one of the following:',
+    -variable => \$choice,
+    -choices =>
+      [
+        {
+         -title => 'choice 1',
+         -value => 1,
+        },
+        {
+         -title => 'choice two, default this one selected',
+         -value => 'two',
+         -selected => 1,
+        },
+      ],
+    );
+
+=back
+
+=cut
+
+sub addSingleChoicePage
+  {
+  my $self = shift;
+  # We have to make a copy of our args in order for them to get
+  # "saved" in this coderef:
+  my $args = {@_};
+  # print STDERR " DDD addSingleChoicePage args are ", Dumper($args);
+  return $self->addPage( sub { $self->_page_single_choice($args) } );
+  } # addSingleChoicePage
+
+sub _page_single_choice
+  {
+  my $self = shift;
+  my $args = shift;
+  if (! defined($args->{-choices}))
+    {
+    croak "-choices argument missing";
+    }
+  if (! ref($args->{-choices}) || (ref($args->{-choices}) ne 'ARRAY'))
+    {
+    croak "-choices must be a ref to an array!";
+    } # if
+  croak "-variable argument missing" if ! defined($args->{-variable});
+  croak "-variable must be a reference!" if ! ref $args->{-variable};
+  # Take care of the -title, -text, etc.:
+  my $frame = $self->blank_frame(%$args);
+  my $content = $frame->Frame(
+                              -background => $self->{background},
+                             )->pack(
+                                     -side => 'top',
+                                     -anchor => "n",
+                                     -padx => 10,
+                                     -pady => 10,
+                                    );
+  my $iNotFirst = 0;
+  foreach my $opt ( @{ $args->{-choices} } )
+    {
+    if (reftype($opt) ne 'HASH')
+      {
+      croak "Option in -choices array must be a hash"
+      } # if
+    $opt->{-title} ||= '';
+    my $sValue = $opt->{-value} || $opt->{-title};
+    my $b = $content->Radiobutton(
+                                  -text     => $opt->{-title},
+                                  -justify  => 'left',
+                                  -relief   => $args->{-relief} || 'flat',
+                                  -font     => "RADIO_BOLD",
+                                  -variable => $args->{-variable},
+                                  -value    => $sValue,
+                                  -background => $self->{background},
+                                  -activebackground => $self->{background},
+                                 )->pack(qw/-side top -anchor w /);
+    ${$args->{-variable}} = $sValue if ! $iNotFirst++;
+    ${$args->{-variable}} = $sValue if $opt->{-selected};
+    my $s = $opt->{-subtitle} || '';
+    if ($s ne '')
+      {
+      # Seven spaces indentation is perfect with my Windows XP default
+      # font:
+      $s =~ s!(^|\n)!$1       !g;
+      my $l = $content->Label(
+                              -font => $self->{defaultFont},
+                              -text => $s,
+                              -anchor => 'w',
+                              -justify  => 'left',
+                              -background => $self->{background},
+                             )->pack(qw/-side top -anchor w/);
+      DEBUG_FRAME && $l->configure(-background => 'light blue');
+      } # if
+    } # foreach
+  return $frame;
+  } # _page_single_choice
+
+
 =head2 Show
 
   $wizard->Show();
@@ -2412,19 +2548,7 @@ sub callback_dirSelect
                                   );
     if ( lc $button eq 'yes' )
       {
-      eval { File::Path::mkpath($$var) };
-      if ($@)
-        {
-        $self->messageBox(
-                          -icon  => 'warning',
-                          -type  => 'ok',
-                          -title => qq'$sDir Could Not Be Created',
-                          -message => "The $sdir you entered could not be created ($@)
-Please choose a different $sdir and press Next to continue."
-                         );
-        return 0;
-        } # if error during mkpath
-      return 1;
+      return $self->_cb_try_create_dir($$var);
       } # if user clicked "Yes"
     $self->messageBox(
                       -icon  => 'info',
@@ -2438,6 +2562,33 @@ Please choose a different $sdir and press Next to continue."
   return 1;
   } # callback_dirSelect
 
+
+sub _cb_try_create_dir
+  {
+  my $self = shift;
+  my $sDirToCreate = shift;
+  my $rasError;
+  File::Path::mkpath($sDirToCreate, {error => \$rasError});
+  # print STDERR Dumper($rasError);
+  if (@$rasError)
+    {
+    my $rh = shift @$rasError;
+    # Only report the first error encountered:
+    my ($sDirEntered) = keys %$rh;
+    my ($sError) = values %$rh;
+    my $sMsg = "The $sdir you entered ($sDirEntered) could not be created ($sError)\nPlease choose a different $sdir.";
+    $self->messageBox(
+                      -icon  => 'warning',
+                      -type  => 'ok',
+                      -title => qq'$sDir Could Not Be Created',
+                      -message => $sMsg,
+                     );
+    return 0;
+    } # if error during mkpath
+  return 1;
+  } # _cb_try_create_dir
+
+
 =head1 ACTION EVENT HANDLERS
 
 A Wizard is a series of pages that gather information and perform
